@@ -1,7 +1,7 @@
 package App::short;
 
-our $DATE = '2015-03-20'; # DATE
-our $VERSION = '0.02'; # VERSION
+our $DATE = '2015-03-21'; # DATE
+our $VERSION = '0.03'; # VERSION
 
 use 5.010001;
 use strict;
@@ -137,12 +137,14 @@ sub _validate {
 
     my @caller = caller(1);
     my $func = $caller[3]; $func =~ s/.+:://;
-    if (1) { #$func eq 'add_short') {
-        if (defined $args->{long}) {
-            return [400, "Invalid long name"] if $args->{long} =~ m![/\\]!;
-        }
-        if (defined $args->{short}) {
-            return [400, "Invalid short name"] if $args->{short} =~ m![/\\]!;
+    if (defined $args->{long}) {
+        return [400, "Invalid long name"] if $args->{long} =~ m![/\\]!;
+    }
+    if (defined $args->{short}) {
+        my @shorts = ref($args->{short}) eq 'ARRAY' ?
+            @{$args->{short}} : ($args->{short});
+        for (@shorts) {
+            return [400, "Invalid short name '$_'"] if m![/\\]!;
         }
     }
 
@@ -164,6 +166,11 @@ $SPEC{list_shorts} = {
             schema => 'bool',
             tags => ['category:filtering'],
         },
+        query => {
+            schema => 'str*',
+            tags => ['category:filtering'],
+            pos => 0,
+        },
     },
 };
 sub list_shorts {
@@ -173,6 +180,8 @@ sub list_shorts {
 
     my $S = $args{short_dir};
 
+    my $q = lc($args{query} // '');
+
     my @res;
     opendir my($dh), $S or
         return [500, "Can't open dir $S: $!"];
@@ -192,6 +201,9 @@ sub list_shorts {
         if (defined $args{broken}) {
             next if $args{broken} xor $broken;
         }
+        if (length($q)) {
+            next unless index(lc($target), $q) >= 0 || index(lc($ent), $q) >= 0;
+        }
 
         push @res, {
             name => $ent,
@@ -380,6 +392,45 @@ sub add_short {
     [200, "OK"];
 }
 
+$SPEC{rm_short} = {
+    v => 1.1,
+    args => {
+        %common_args,
+        short => {
+            schema => ['array*', of=>'str*', min_len=>1],
+            req => 1,
+            pos => 0,
+            greedy => 1,
+            element_completion => $_completion_short,
+        },
+    },
+};
+sub rm_short {
+    require Perinci::Object;
+
+    my %args = @_;
+    my $res = _validate(\%args);
+    return $res unless $res->[0] == 200;
+
+    my $S = $args{short_dir};
+
+    my $envres = Perinci::Object::envresmulti();
+
+    for my $s (@{ $args{short} }) {
+        my $path = "$S/$s";
+
+        if (!(-l $path)) {
+            $envres->add_result(404, "Short name not found", {item_id=>$s});
+        } elsif (!unlink($path)) {
+            $envres->add_result(500, "Can't unlink: $!", {item_id=>$s});
+        } else {
+            $envres->add_result(200, "OK", {item_id=>$s});
+        }
+    }
+
+    $envres->as_struct;
+}
+
 1;
 # ABSTRACT: Manage short directory symlinks
 
@@ -395,7 +446,7 @@ App::short - Manage short directory symlinks
 
 =head1 VERSION
 
-This document describes version 0.02 of App::short (from Perl distribution App-short), released on 2015-03-20.
+This document describes version 0.03 of App::short (from Perl distribution App-short), released on 2015-03-21.
 
 =head1 SYNOPSIS
 
@@ -532,6 +583,36 @@ Arguments ('*' denotes required arguments):
 
 =item * B<long_include> => I<array[str]>
 
+=item * B<query> => I<str>
+
+=item * B<short_dir>* => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 rm_short(%args) -> [status, msg, result, meta]
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<long_dir>* => I<str>
+
+=item * B<long_include> => I<array[str]>
+
+=item * B<short>* => I<array[str]>
+
 =item * B<short_dir>* => I<str>
 
 =back
